/*
 * XDPSlib.h - client header file for DPS library.  This file describes the
 * Xlib interface to the Display Postscript extension.  It should be used only
 * by that interface and by code that actually uses this interface (e.g., the
 * client library interface).
 
  Copyright (c) 1988, 1989 Adobe Systems Incorporated,
  and Digital Equipment Corporation, Maynard, Massachusetts
  All rights reserved.
  
 *
 * Author:	Terry Weissman & Ramin Behtash
 * 		Digital & Adobe
 */


#ifndef _XDPSlib_h
#define _XDPSlib_h

#include "Xutil.h"
#include "XDPS.h"

/*
 * Default standard colormap type.
 */

#define DefaultStdCMap	((XStandardColormap *) -1)


/*
 * event masks defined for convenience
 */

#define XDPSL_ALL_EVENTS \
 (PSRUNNINGMASK|PSNEEDSINPUTMASK|PSZOMBIEMASK|PSFROZENMASK)

#ifdef __cplusplus			/* do not leave open across includes */
extern "C" {					/* for C++ V2.0 */
#endif

/*
 * XDPSLInit returns the first event number for the extension.
 */

extern int XDPSLInit(
#if NeedFunctionPrototypes
  Display *,
  int *numberType,		/* RETURN */
  char **floatingName		/* RETURN */
#endif
);

extern ContextXID XDPSLCreateContext(
#if NeedFunctionPrototypes
  Display *,
  SpaceXID sxid,
  Drawable drawable,
  GC gc,
  int x,
  int y,
  unsigned int eventMask,
  XStandardColormap *grayRamp,
  XStandardColormap *colorCube,
  unsigned short actual,
  ContextPSID *cpsid		/* RETURN */
#endif
);

extern ContextXID XDPSLCreateContextAndSpace(
#if NeedFunctionPrototypes
  Display *,
  Drawable drawable,
  GC gc,
  int x,
  int y,
  unsigned int eventMask,
  XStandardColormap *grayRamp,
  XStandardColormap *colorCube,
  unsigned short actual,
  ContextPSID *cpsid,		/* RETURN */
  SpaceXID *sxid		/* RETURN */
#endif
);
extern SpaceXID XDPSLCreateSpace(
#if NeedFunctionPrototypes
  Display *
#endif
);

extern void XDPSLGiveInput(
#if NeedFunctionPrototypes
  Display *,
  ContextXID cxid,
  char *data,
  int length
#endif
);

extern int XDPSLGetStatus(
#if NeedFunctionPrototypes
  Display *,
  ContextXID cxid
#endif
);

extern void XDPSLDestroySpace(
#if NeedFunctionPrototypes
  Display *,
  SpaceXID sxid
#endif
);

extern void XDPSLReset(
#if NeedFunctionPrototypes
  Display *,
  ContextXID cxid
#endif
);

extern void XDPSLNotifyContext(
#if NeedFunctionPrototypes
  Display *,
  ContextXID cxid,
  int type
#endif
);

extern ContextXID XDPSLCreateContextFromID(
#if NeedFunctionPrototypes
  Display *,
  ContextPSID cpsid,
  SpaceXID *sxid		/* RETURN */
#endif
);


extern Status XDPSLIDFromContext(
#if NeedFunctionPrototypes
  Display *,
  ContextPSID cpsid,
  ContextXID *cxid,		/* RETURN */
  SpaceXID *sxid		/* RETURN */
#endif
);
/* Returns 1 on success, 0 on failure (cpsid not a valid context). */

extern ContextPSID XDPSLContextFromXID(
#if NeedFunctionPrototypes
  Display *,
  ContextXID cxid
#endif
);
  
extern void XDPSLSetStatusMask(
#if NeedFunctionPrototypes
  Display *,
  ContextXID cxid,
  unsigned long enableMask,
  unsigned long disableMask,
  unsigned long nextMask
#endif
);

typedef struct _XDPSLOutputEvent {
    int type;			/* of event. */
    unsigned long serial;	/* # of last request processed by server. */
    Bool send_event;		/* True if this came from a SendEvent req. */
    Display *display;		/* Display the event was read from. */
    ContextXID cxid;		/* Context from which data comes. */
    int length;			/* Number of characters of data. */
    char data[24];		/* Data received. */
} XDPSLOutputEvent;


typedef struct _XDPSLStatusEvent {
    int type;			/* of event. */
    unsigned long serial;	/* # of last request processed by server. */
    Bool send_event;		/* True if this came from a SendEvent req. */
    Display *display;		/* Display the event was read from. */
    ContextXID cxid;		/* Context whose status we're reporting. */
    int status;			/* Status of context. */
} XDPSLStatusEvent;
    
extern void XDPSLSetTextEventHandler(
#if NeedFunctionPrototypes
  Display *,
#ifdef NESTED_FUNC_PROTO_COMPILER_BUG_FIXED
  void (*proc)(XDPSLOutputEvent *event)
#else
  void (*proc)(XDPSLOutputEvent *)
#endif
#endif
);

extern void XDPSLSetStatusEventHandler(
#if NeedFunctionPrototypes
  Display *,
#ifdef NESTED_FUNC_PROTO_COMPILER_BUG_FIXED
  void (*proc)(XDPSLStatusEvent *event)
#else
  void (*proc)(XDPSLStatusEvent *)
#endif
#endif
);

#ifdef __cplusplus
}						/* for C++ V2.0 */
#endif

#endif _XDPSlib_h
