#ifndef __STDIO_H__
#define __STDIO_H__
#ifdef __cplusplus
extern "C" {
#endif
/* Copyright (C) 1989,1990 Silicon Graphics, Inc. All rights reserved.*/
/* ------------------------------------------------------------------ */
/* | Copyright Unpublished, MIPS Computer Systems, Inc.  All Rights | */
/* | Reserved.  This software contains proprietary and confidential | */
/* | information of MIPS and its suppliers.  Use, disclosure or     | */
/* | reproduction is prohibited without the prior express written   | */
/* | consent of MIPS.                                               | */
/* ------------------------------------------------------------------ */
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved 	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*#ident "$Revision: 1.1 $" */

/* ANSI C Notes:
 *
 * - THE IDENTIFIERS APPEARING OUTSIDE OF #ifdef __EXTENSIONS__ IN THIS
 *   standard header ARE SPECIFIED BY ANSI!  CONFORMANCE WILL BE ALTERED
 *   IF ANY NEW IDENTIFIERS ARE ADDED TO THIS AREA UNLESS THEY ARE IN ANSI's
 *   RESERVED NAMESPACE. (i.e., unless they are prefixed by __[a-z] or
 *   _[A-Z].  For external objects, identifiers with the prefix _[a-z] 
 *   are also reserved.)
 *
 * - Additional reserved identifiers include FILE, fpos_t, BUFSIZ,
 *   EOF, FOPEN_MAX, FILENAME_MAX, L_tmpnam, SEEK_CUR, SEEK_END, SEEK_SET,
 *   TMP_MAX, stderr, stdout, stdin, size_t
 *
 * - No stdio functions may be implemented as macros with the exception
 *   of getc and putc.  As getchar and putchar are equivalent to
 *   getc(stdin) and putc(c,stdin), respectively, they evidently may
 *   also be expressed as macros.  In particular, it is NOT legal
 *   to express fgetc or fputc as macros.
 */

#ifndef _SIZE_T_
#define _SIZE_T_
typedef unsigned size_t;
#endif /* !_SIZE_T_ */

typedef unsigned fpos_t;	/* The type capable of recording all
				 * information needed to specify uniquely
				 * every position within a file. */
typedef struct __file_s {
	int	_cnt;
	unsigned char	*_ptr;
	unsigned char	*_base;
	char	_flag;
	char	_file;
} FILE;

/*
 * FILE _flag values:
 *
 *  _IONBF, _IOLBF and _IOFBF are also used as the "type" argument to setvbuf.
 */
#define _IOFBF		0000	/* fully buffered */
#define _IONBF		0004	/* not buffered */
#define _IOLBF		0100	/* line buffered */

#define _IOREAD		0001
#define _IOWRT		0002
#define _IOMYBUF	0010
#define _IOEOF		0020
#define _IOERR		0040
#define _IORW		0200

/* Buffer size for multi-character output to unbuffered files. */
#define _SBFSIZ 8

#define	BUFSIZ	8192

#ifndef NULL
#define NULL	0
#endif
#define EOF	(-1)

extern unsigned char *	_bufendtab[];


#if defined(__EXTENSIONS__)
/* the macro versions of _bufend() and _bufsiz() are not allowed in pure ANSI 
 * environments as their replacements contain identifiers which are not 
 * reserved in all contexts.
 */
#define _bufend(p)	_bufendtab[(p)->_file]
#define _bufsiz(p)	(_bufend(p) - (p)->_base)
/* in impure environments we can use macro replacements with _iob as well */
#define stdin		(&_iob[0])
#define stdout		(&_iob[1])
#define stderr		(&_iob[2])
#else /* __EXTENSIONS__ */
/* in pure ANSI environments, the identifier _iob cant be used in the 
   replacement of stdin, stdout, and stderr.  Special variables have
   been introduced in libc which have been initialized appropriately.
   (e.g., __stdin has been initialized with the value &_iob[0])
*/
extern FILE *__stdin, *__stdout, *__stderr;
#define stdin		(__stdin)
#define stdout		(__stdout)
#define stderr		(__stderr)
#endif /* __EXTENSIONS__ */

#ifndef SEEK_SET
/*
 * SEEK_{CUR,END,SET} are integral constants suitable for use as the
 * third argument of the fseek(3S) function.  They are protected from multiple
 * definition as they occur in the non-standard unistd.h.
 */
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif /* !SEEK_SET */

/*
 * FOPEN_MAX is the ANSI definition for the minimum number of files the
 * installation guarantees can be open simultaneously.
 */
#define FOPEN_MAX	100
#define _NFILE		FOPEN_MAX

/*
 * FILENAME_MAX is the maximum length for a file name string that
 * the implementation guarantees can be opened.
 */
#define FILENAME_MAX	255	/* NAME_MAX from limits.h */

/*
 * TMP_MAX is the minimum number of unique file names which shall
 * be generated by the tmpnam() and _mktemp() functions.
 */
#define TMP_MAX	17576	/* see tmpnam(3C) */

#define _P_tmpdir	"/usr/tmp/"
#define L_tmpnam	(sizeof(_P_tmpdir) + 15)



extern FILE	_iob[];
extern FILE *	tmpfile(void);
extern char *	tmpnam(char *);
extern int	fclose(FILE *);
extern int	fflush(FILE *);
extern FILE *	fopen(const char *, const char *);
extern FILE *	freopen(const char *, const char *, FILE *);
extern void	setbuf(FILE *, char *);
extern int	setvbuf(FILE *, char *, int, size_t);
extern int	fprintf(FILE *, const char *, ...);
extern int	fscanf(FILE *, const char *, ...);
extern int	printf(const char *, ...);
extern int	scanf(const char *, ...);
extern int	sprintf(char *, const char *, ...);
extern int	sscanf(const char *, const char *, ...);
extern int	vfprintf(FILE *, const char *, /* va_list */ char *);
extern int	vprintf(const char *, /* va_list */ char *);
extern int	vsprintf(char *, const char *, /* va_list */ char *);
extern int	fgetc(FILE *);
extern char *	fgets(char *, int, FILE *);
extern int	fputc(int, FILE *);
extern int	fputs(const char *, FILE *);
extern char *	gets(char *);
extern int	puts(const char *);
extern int	ungetc(int, FILE *);
extern size_t	fread(void * , size_t , size_t , FILE *);
extern size_t	fwrite(const void *, size_t, size_t, FILE *);
extern int	fseek(FILE *, long int, int);
extern long int	ftell(FILE *);
extern void	rewind(FILE *);
extern void	perror(const char *);
extern int	_flsbuf(unsigned,FILE*);
extern int	_filbuf(FILE*);
extern int	getc(FILE *);
extern int 	getchar(void);
extern int 	putc(int, FILE *);
extern int	putchar(int);
extern int	_semputc(int, FILE *);
extern int	_semgetc(FILE *);
extern int	feof(FILE *);
extern int	ferror(FILE *);
extern void	clearerr(FILE *);
extern int	fgetpos(FILE *, fpos_t *);
extern int	fsetpos(FILE *, const fpos_t *);

/* ANSI version of standard stdio functions whose 'usual' names are
   not reserved.
*/
extern char *	_mktemp(char *);
extern int	_fileno(FILE *);

#if defined(__STDC__) || defined(_LANGUAGE_C_PLUS_PLUS)
extern int	remove(const char *);
extern int	rename(const char *, const char *);
#endif

#ifndef _LINT

/* NOTE:
 *   The getc and putc macros have been modified so that processes in
 *   a share group (see sproc(2)) can use them simultaneously.
 */
extern void *_sproced;
extern int _us_rsthread_stdio;

#define clearerr(p)	((void) ((p)->_flag &= ~(_IOERR | _IOEOF)))
#define feof(p)		((p)->_flag & _IOEOF)
#define ferror(p)	((int) (p)->_flag & _IOERR)

#if defined(__EXTENSIONS__)
/* remove the macro versions of getc() and putc() in pure ANSI environments
 * as their replacements contain identifiers which are not reserved 
 * in all contexts.
 */
#define getc(p)		( _us_rsthread_stdio ? _semgetc(p) : \
			((--(p)->_cnt > 0 || \
	     		(((p)->_cnt == 0 ) && ((p)->_ptr != (p)->_base)))?\
			(int) *(p)->_ptr++:\
			((p)->_cnt == 0 ? (int) *(p)->_ptr:_filbuf(p))))
#define putc(x, p)	( _us_rsthread_stdio ? _semputc(x,p) : \
			(--(p)->_cnt < 0 ? _flsbuf((unsigned char) (x), (p)) : \
			(int) (*(p)->_ptr++ = (unsigned char) (x))))
#endif /* __EXTENSIONS__ */

#define getchar()	getc(stdin)
#define putchar(x)	putc((x), stdout)
#define _fileno(p)	(p)->_file

#if (defined(__EXTENSIONS__) || defined(_POSIX_SOURCE))
#define fileno(p)	(p)->_file
#endif /* __EXTENSIONS__ || _POSIX_SOURCE */

#endif /* !_LINT */

#if (defined (__EXTENSIONS__) || defined(_POSIX_SOURCE))

extern FILE *	fdopen(int, const char *);
extern FILE *	popen(const char *, const char *);
extern char *	ctermid(char *);
extern char *	cuserid(char *);
extern int	pclose(FILE *);
extern int	system(const char *);	/* also defined in stdlib.h */

#define L_ctermid	9
#define L_cuserid	9
#define P_tmpdir	_P_tmpdir

#if !defined(_POSIX_SOURCE)
extern char *	tempnam(const char *, const char *);
extern char *	mktemp(char *);
extern int	mkstemp(char *);
extern int	getw(FILE *);
extern int	putw(int, FILE *);
extern int	setbuffer(FILE *, char *, int);
extern int	setlinebuf(FILE *);
#endif /* !_POSIX_SOURCE */
#endif /* __EXTENSIONS__ || _POSIX_SOURCE */

#if (defined(__EXTENSIONS__) && defined(_LANGUAGE_C_PLUS_PLUS))
extern void	exit(int);		/* also defined in stdlib.h */
extern double	atof(const char *);	/* also defined in stdlib.h */
extern int	errno;
extern char *	sys_errlist[];
#endif /* _LANGUAGE_C_PLUS_PLUS */

#ifdef __cplusplus
}
#endif
#endif /* !__STDIO_H__ */
