#ifndef __STDLIB_H__
#define __STDLIB_H__
#ifdef __cplusplus
extern "C" {
#endif
/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*#ident "$Revision: 1.12 $" */

/* ANSI C Notes:
 *
 * - THE IDENTIFIERS APPEARING OUTSIDE OF #ifdef __EXTENSIONS__ IN THIS
 *   standard header ARE SPECIFIED BY ANSI!  CONFORMANCE WILL BE ALTERED
 *   IF ANY NEW IDENTIFIERS ARE ADDED TO THIS AREA UNLESS THEY ARE IN ANSI's
 *   RESERVED NAMESPACE. (i.e., unless they are prefixed by __[a-z] or
 *   _[A-Z].  For external objects, identifiers with the prefix _[a-z] 
 *   are also reserved.)
 *
 * - Functions whose names begin with 'str' followed by a lower-case
 *   letter may be added to stdlib.h, and are thus reserved.
 *
 * - The header <stdlib.h> "was invented by the Committee to hold an assortment
 *   of functions that were otherwise homeless."  (Rationale, sec 4.10)
 */

#ifndef	NULL
#define NULL	0
#endif

/* Arguments to exit(2). */
#define EXIT_FAILURE	1
#define EXIT_SUCCESS	0

/* Maximum value returned by rand(3C). */
#define RAND_MAX	32767

/* Maximum # of bytes in a multibyte character. */
#define MB_CUR_MAX	1

#ifdef _LANGUAGE_ASSEMBLY
/* assembly-level offsets of quot, rem fields into [l ]div_t structure below */
#define _QUOT_OFFSET 	0
#define _REM_OFFSET	4
#define _LQUOT_OFFSET 	0
#define _LREM_OFFSET	4
#endif 

#if (defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS))

/* Type of the value returned by div() function. */
/* NOTE!! IF YOU UPDATE THIS STRUCTURE, UPDATE THE OFFSETS 
	AT THE BEGINNING OF <stdlib.h> */
struct __div_s {
	int quot, rem;
};
typedef struct __div_s div_t;

/* Type of the value returned by ldiv() function */
/* NOTE!! IF YOU UPDATE THIS STRUCTURE, UPDATE THE OFFSETS 
	AT THE BEGINNING OF <stdlib.h> */
struct __ldiv_s {
	long int quot, rem;
};
typedef struct __ldiv_s ldiv_t;

#ifndef _SIZE_T_
#define _SIZE_T_
typedef unsigned size_t;
#endif /* !_SIZE_T_ */

#ifndef _WCHAR_T_
#define _WCHAR_T_
typedef unsigned char wchar_t;
#endif /* !_WCHAR_T_ */


extern double	atof(const char * );
extern int	atoi(const char *);
extern long int atol(const char *);
extern double	strtod(const char *, char **);
extern long int strtol(const char *, char **, int);
extern unsigned long int strtoul(const char *, char **, int);
extern int	rand(void);
extern void	srand(unsigned int);
extern void *	calloc(size_t, size_t);
extern void *	malloc(size_t);
extern void	free(void *);
extern void *	realloc(void *, size_t);
extern void	abort(void);
extern void	exit(int);
extern char *	getenv(const char *);
extern int	system(const char *);
extern void *	bsearch(const void *, const void *, size_t, size_t,
			 int (*) (const void *, const void *));
extern void	qsort(void *, size_t, size_t,
			 int (*) (const void *, const void *));
extern int	abs(int);
extern int	atexit(void (*) (void));
extern div_t	div(int, int);
extern long int labs(long int);
extern ldiv_t	ldiv(long int, long int);

extern int	mblen(const char *, size_t);
extern int	mbtowc(wchar_t *, const char *, size_t);
extern int	wctomb(char *, wchar_t);
extern size_t	mbstowcs(wchar_t *, const char *, size_t);
extern size_t	wcstombs(char *, const wchar_t *, size_t);

#if defined(__EXTENSIONS__) || defined(_POSIX_SOURCE)
extern int putenv(const char *);
#endif /*defined(__EXTENSIONS__) || defined(_POSIX_SOURCE) */

#endif /*(defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)) */

#ifdef __cplusplus
}
#endif
#endif /* !__STDLIB_H__ */
