#ifndef __NFS_DEBUG_HEADER__
#define	__NFS_DEBUG_HEADER__
/*
 * nfs debugging support
 */
#if defined _KERNEL && defined SVR3
#include "sys/debug.h"
#endif

#ifndef NFSDEBUG
#ifdef	_KERNEL

#undef	DBG
#define	DBG(flag,args)

#endif	/* _KERNEL */
#else	/* NFSDEBUG */

#define	NFSDBG_GETATTR	0x00000001	/* getattr: procedure 1, v2 */
#define	NFSDBG_SETATTR	0x00000002	/* setattr: procedure 2, v2 */
#define	NFSDBG_LOOKUP	0x00000004	/* lookup: procedure 4, v2 */
#define	NFSDBG_READLINK	0x00000008	/* readlink: procedure 5, v2 */
#define	NFSDBG_READ	0x00000010	/* read: procedure 6, v2 */
#define	NFSDBG_WRITE	0x00000020	/* write: procedure 8, v2 */
#define	NFSDBG_CREATE	0x00000040	/* create: procedure 9, v2 */
#define	NFSDBG_REMOVE	0x00000080	/* remove: procedure 10, v2 */
#define	NFSDBG_RENAME	0x00000100	/* rename: procedure 11, v2 */
#define	NFSDBG_LINK	0x00000200	/* link: procedure 12, v2 */
#define	NFSDBG_SYMLINK	0x00000400	/* link: procedure 13, v2 */
#define	NFSDBG_MKDIR	0x00000800	/* mkdir: procedure 14, v2 */
#define	NFSDBG_RMDIR	0x00001000	/* rmdir: procedure 15, v2 */
#define	NFSDBG_READDIR	0x00002000	/* readdir: procedure 16, v2 */
#define	NFSDBG_STATFS	0x00004000	/* statfs: procedure 17, v2 */
#define	NFSDBG_ALLPROCS	0x00007fff	/* all 15 supported procedures */
#define	NFSDBG_OPEN	0x00008000	/* open remote file hook */
#define	NFSDBG_CLOSE	0x00010000	/* close remote file hook */
#define	NFSDBG_READRPC	0x00020000	/* actual read rpc, maybe from biod */
#define	NFSDBG_WRITERPC	0x00040000	/* actual write rpc */
#define	NFSDBG_ACCESS	0x00080000	/* access system call hook */
#define	NFSDBG_IPUT	0x00100000	/* inode caching/recycling */
#define	NFSDBG_BMAP	0x00200000	/* map fs blocks to basic blocks */
#define	NFSDBG_STRATEGY	0x00400000	/* start i/o on an rnode */
#define	NFSDBG_BIO	0x00800000	/* finish i/o on an rnode */
#define	NFSDBG_MOUNT	0x01000000	/* mount vfs op */
#define	NFSDBG_UNMOUNT	0x02000000	/* unmount vfs op */
#define	NFSDBG_IREAD	0x04000000	/* FSS inode cache fill hook */

#define	NFSDBG_UTIL	0x08000000	/* low-level utility functions */
#define	NFSDBG_CLIENT	0x10000000	/* trace client side ops */
#define	NFSDBG_SERVER	0x20000000	/* trace server functions */
#define	NFSDBG_XDR	0x40000000	/* trace xdr (client and server) */
#define	NFSDBG_RETURN	0x80000000	/* trace procedure exit points */
#define	NFSDBG_ALL	0xffffffff

#ifdef _KERNEL

/*
 * If nfs_dbgmask is zero, print if any flag bits are also in nfs_dbgopts.
 * Otherwise if nfs_dbgopts is non-zero, mask flag with nfs_dbgmask and
 * print if the result is exactly nfs_dbgopts.
 */
#undef	DBG
#define	DBG(flag,args) \
	if (nfs_dbgmask == 0 && ((flag) & nfs_dbgopts) \
	    || nfs_dbgopts && ((flag) & nfs_dbgmask) == nfs_dbgopts) { \
		nfs_dbgprintf args; \
	}

extern u_long	nfs_dbgmask;
extern u_long	nfs_dbgopts;
extern void	nfs_dbgprintf();

#endif	/* _KERNEL */
#endif	/* NFSDEBUG */
#endif	/* __NFS_DEBUG_HEADER__ */
