#ifndef __SYS_GMHOST_H__
#define __SYS_GMHOST_H__
/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

/*
|| Definitions for Host/Graphics Manager interface
||
|| $Revision: 3.38 $
*/

/*************** Commands sent from the host to the GM ***************/

#define     GM_ABORT			0		/* abort current req */
#define	    GM_RESET		    	44
#define	    GM_SETMODE			45
#define	       GM_MODE_CONSOLE			0
#define	       GM_MODE_HOST			1
#define	       GM_MODE_UART_0			2
#define	       GM_MODE_UART_1			3
#define	    GM_PIXEL_DMA		46
#define	       GM_PIXEL_DMA_READ		0
#define	       GM_PIXEL_DMA_WRITE		1
#define	    GM_PUTSTR			47
#define	    GM_LOAD_UCODE		48
#define	    GM_EXECUTE			49
#define	       GM_DEST_GM			1
#define	       GM_DEST_PP			2
#define	       GM_DEST_GE1			3
#define	       GM_DEST_GE2			GM_DEST_GE1+1
#define	       GM_DEST_GE3			GM_DEST_GE1+2
#define	       GM_DEST_GE4			GM_DEST_GE1+3
#define	       GM_DEST_GE5			GM_DEST_GE1+4
#define	    GM_ACTIVATE			50
#define	    GM_DIAG_CMD			51
#define	    GM_RESET_TEXT		52
#define	    GM_OUTSTR			53
#define     GM_INITMEMORY		54
#define     GM_GENERIC_CMD		55

#ifndef ASM68K
/*
|| Host to GM communication area
*/
typedef struct host_to_gm_comm_s {
	int busy;	/* synchronization flag */
	int cmd;	/* Host to GM command */
	int len;	/* length in bytes of command specific data */
	union htgc_un {
	/* define maximum size of data area */
		char	data[2048 - 3 * sizeof (int)];
	
	/*
	|| The rest of the structures in this union define the
	|| command specific data for each host to GM command.
	*/

	/* GM_SETMODE */
		struct gm_setmode_s {
			int mode;
		} gm_setmode_u;

	/* GM_PUTCHAR */
		struct gm_putchar_s {
			char str;
		} gm_putchar_u;

	/* GM_PUTSTR */
		struct gm_putstr_s {
			char str[1];
		} gm_putstr_u;

	/* GM_LOAD */
		struct gm_load_s {
			int dest;
			int addr;
			char data[sizeof (int)];
		} gm_load_u;

	/* GM_EXECUTE */
		struct gm_execute_s {
			int dest;
			int addr;
			int argc;
			int argv[1];
		} gm_execute_u;

	/* GM_DIAG_CMD */
		struct gm_diag_cmd_s {
			int cmd;
			int len;
			char data[sizeof (int)];
		} gm_diag_cmd_u;

	/* GM_RESET_TEXT */
		struct gm_reset_text_s {
			long drawlogo;	/* TRUE if we should draw sgi logo */
		} gm_reset_text_u;

	/* GM_OUTSTR */
		struct gm_outstr_s {
			char str[1];	/* must be a NULL terminated string */
		} gm_outstr_u;

	/* GM_PIXEL_DMA */
		struct gm_pixel_dma_s {
			int optype;	/* 0 = read, 1 = write */
			char *buffer;	/* MP bus physical addr of user buf */
			int buflen;	/* transfer length in bytes */
			int offset;	/* offset into pixel buffer in bytes */
		} gm_pixel_dma_u;
	} un;
}   HOST_TO_GM_INT;

typedef struct host_to_gm_comm_s host_to_gm_comm_t;

/*
|| Convenient defines for referencing command specific data
*/
#define gm_setmode	un.gm_setmode_u
#define gm_putchar	un.gm_putchar_u
#define gm_putstr	un.gm_putstr_u
#define gm_load		un.gm_load_u
#define gm_execute	un.gm_execute_u
#define gm_diag_cmd	un.gm_diag_cmd_u
#define gm_reset_text	un.gm_reset_text_u
#define gm_outstr	un.gm_outstr_u
#define gm_pixel_dma	un.gm_pixel_dma_u

/*
|| Derived size constants
*/
#define _H2GM_(p)		((struct host_to_gm_comm_s *)(p))
#define GM_LOAD_DATA_SIZE	(sizeof (union htgc_un) \
					- sizeof (struct gm_load_s) \
					+ sizeof (_H2GM_(0)->gm_load.data))
#define GM_EXEC_DATA_SIZE	(sizeof (union htgc_un) \
					- sizeof (struct gm_execute_s) \
					+ sizeof (_H2GM_(0)->gm_execute.argv))
#define GM_PUTSTR_DATA_SIZE	(sizeof (union htgc_un))
#define GM_DIAG_DATA_SIZE	(sizeof (union htgc_un) \
					- sizeof (struct gm_diag_cmd_s) \
					+ sizeof (_H2GM_(0)->gm_diag_cmd.data))
#define GM_OUTSTR_DATA_SIZE	(sizeof (union htgc_un))

/*
|| Structure for GM to HOST communication area
*/
typedef struct gm_to_host_comm_s {
	int busy;	/* synchronization flag */
	int cmd;	/* GM to Host command */
	int len;	/* length in bytes of command specific data */
	union gthc_un {
	/* define maximum size of data area */
		char	data[2048 - 3 * sizeof (int)];

	/* HOST_PUTCHAR */
		struct host_putchar_s {
			char str;
		} host_putchar_u;
	
	/* HOST_ERROR */
		struct host_error_s {
			int context;
			int cause;
		} host_error_u;

	/* HOST_ACKNOWLEDGE */
		struct host_acknowledge_s {
			int cmd;	/* command being acknowledged */
			int status;	/* 0 if OK, < 0 if error */
		} host_acknowledge_u;

	/* HOST_SWAPBUFFERSCOMPLETE */
#define contextmask widmask		/* more forgiving backward compatibility */
		struct host_swapbuffers_s {
			ulong widmask;	/* wids that swapped (bitmask) */
			ulong swaptime;	/* when they swapped */
		} host_swapbuffers_u;
	/* HOST_DIAG_CMD */
		struct host_diag_cmd_s {
			int cmd;	/* diagnostic command */
			int status;	/* 0 if OK, < 0 if error */
			int len;	/* length in bytes of data */
			char data[sizeof (int)];
		} host_diag_cmd_u;
	
	/* HOST_FEEDBACK */
		struct host_feedback_s {
			int context;	/* hardware context id */
			int last;	/* if TRUE, this is end of feedback */
		} host_feedback_u;
	
	/* HOST_STORE_CX */
		struct host_store_cx_s {
			int context;	/* hardware context id */
			int len;	/* number of bytes of context data */
		} host_store_cx_u;

	/* HOST_PIXEL_DMA_DONE */
		struct host_pixel_dma_done_s {
			int status;	/* 0 if OK, < 0 if error */
			int len;	/* number of bytes of copied */
		} host_pixel_dma_done_u;

	/* HOST_COPY_DONE */
		struct host_copy_done_s {
			int len;	/* number of bytes copied */
		} host_copy_done_u;

	/* HOST_GSYNCCOMPLETE */
		struct host_gsync_s {
			int id;		/* sequence id for matching up */
		} host_gsync_u;

	/* HOST_VIDEO_PERIPH_ACK */
		struct host_video_periph_s {
			int status;	/* return status for video periph cmd */
		} host_video_periph_u;
	} un;
} GM_TO_HOST_INT;
typedef struct gm_to_host_comm_s gm_to_host_comm_t;

#define host_putchar		un.host_putchar_u
#define host_error		un.host_error_u
#define host_acknowledge	un.host_acknowledge_u
#define host_swapbuffers	un.host_swapbuffers_u
#define host_diag_cmd		un.host_diag_cmd_u
#define host_feedback		un.host_feedback_u
#define host_store_cx		un.host_store_cx_u
#define host_pixel_dma_done	un.host_pixel_dma_done_u
#define host_copy_done		un.host_copy_done_u
#define host_gsync		un.host_gsync_u

/* monitor types found in gm_reset_text.drawlogo << by 8 */
#ifndef MONITOR_60HZ		/* to avoid clashing with gf3vme.h */
#define MONITOR_60HZ 		0x00000000
#define MONITOR_30HZ 		0x00000001
#define MONITOR_NTSC 		0x00000002
#define MONITOR_PAL 		0x00000003
#define MONITOR_343		0x00000004
#define MONITOR_NO_SYNC_ON_GREEN	0x00000010
#define MONITOR_GENLOCK		0x00000020
#endif

/*
|| Derived size constants
*/
#define _GM2H_(p)		((struct gm_to_host_comm_s *)(p))
#define HOST_DIAG_DATA_SIZE	(sizeof (union gthc_un) \
				- sizeof (struct host_diag_cmd_s) \
				+ sizeof (_GM2H_(0)->host_diag_cmd.data))

/* Size of communication area mapped into window manager's addr space */
#define	    HOST_COMM_AREA_SIZE	    2048
/* Size of each context's feedback region in bytes (3 MIPS pages) */
#define	    FEEDBACK_REGION_SIZE    12288

/*
** Every graphics process has a region of the GM's DRAM mapped into
** its address space, for the purpose of feedback.
** The total size of a feedback region is a multiple of the page size,
** the first bytes of which are a feedback_data_struct.
**
** The handshake between the host program and the GM goes as follows:
** The program sets the "ready" flag to FALSE, drops a token into the pipe,
** and then spins waiting for ready to go to TRUE.
** The GM writes the data into the region and sets ready to TRUE.
**
** The GM never examines the ready flag; it assumes that the host
** always finishes with one piece of feedback data before initiating
** the next feedback operation. This guarantees that an errant host
** program cannot hang the GM by trashing the ready flag.
**
** For double-buffered feedback, the feedback region is divided exactly
** in half. Each half begins with its own feedback_data_struct.
** When the host issues such a feedback command, it specifies which
** half of the feedback region is to be filled. It can thus have
** as many as two outstanding feedback requests and still avoid
** having the GM spin on a bit in user address space.
*/

/* XXX - should #define VOLATILE somewhere */
#ifdef m68000
typedef struct feedback_data_struct {
#else
typedef volatile struct	feedback_data_struct {
#endif
    int		ready;	/* TRUE => data is ready to be copied */
    uint	size;	/* Amount of data in buffer */
    unchar	data[1];/* Data immediately follows size field */
} FEEDBACK_DATA,  *P_FEEDBACK_DATA;

#define GM_MAX_SCREENS	4
typedef struct gm_screeninfo_s {
	int xmax;		/* maximum x dimension */
	int ymax;		/* maximum y dimension */
	int spans;		/* number of span processors */
	int noalpha;		/* flag if no alpha memory is available */
} gm_screeninfo_t;
typedef struct gm_dispinfo_s {
	int hwmask;		/* mask indicating what display hw present */
#define GM_HW_SCREEN0	0x00000001	/* screen 0 present */
#define GM_HW_SCREEN1	0x00000002	/* screen 1 present */
#define GM_HW_SCREEN2	0x00000004	/* screen 2 present */
#define GM_HW_SCREEN3	0x00000008	/* screen 3 present */
#define GM_HW_VS1	0x20000000	/* VS1 present */
#define GM_HW_VO1	0x40000000	/* VO1 present */
#define GM_HW_VP1	0x80000000	/* VP1 present */
#define GM_HW_IMP5	0x10000000	/* IMP5 present*/
#define GM_HW_BI	0x08000000	/* Bi-endian capability present */
	int screens;		/* number of primary display screens */
	gm_screeninfo_t screeninfo[GM_MAX_SCREENS]; /* info on primary scrns */
} gm_dispinfo_t;

/*
** Structure of data shared between the host and the GM.
** This area contains state information that needs to be constantly
** available to both sides and needs to be updated quickly.
*/

struct cursorstuff_s {
#define CS_MOD_MODE	0x1	
#define CS_MOD_GLYPH	0x2	
#define CS_MOD_COLORS	0x4	
	char	modified_flags;
	char	cache_id;
	char	mode;
#define GM_CURSOROFF		0x00
#define GM_CROSSHAIRCURSOR	0x38
#define GM_GLYPHCURSOR		0X58
	char	filler;
	int	colors[3];
};

/*
** Structure of data shared between the host and the GM2.
** This area contains state information that needs to be constantly
** available to both sides and needs to be updated quickly.
*/
typedef struct	gm2_shared_s {
	int	gmstate;	/* current state of the GM */
	int	cursorx;	/* current x value of the cursor */
	int	cursory;	/* current y value of the cursor */
	int	textPortReady;	/* TRUE if textPort is able to accept chars */
	int	pickready[16];	/* ready flag per context for pick/select */
	int	scrblank;	/* boolean: screen display blanked */
	int	pipecpu;	/* id of cpu to interrupt (owner of pipe) */
	int	feedp[16];	/* ptrs to feedback areas (1 per context) */
	int	storep;		/* ptr to store context area */
	int	storelim;	/* size in bytes of store context area */
	int	fifofull;	/* GM to host fifo full flag */
	int	cputype;	/* cpu type (ip5 or ip7) */
	int	broadcast;	/* broadcast FIFO interrupt for share groups */
	int	rv_state;	/* rv, dg, vo, vp bits defined in gshm.h */
	gm_dispinfo_t dispinfo;	/* display information (described above) */
	int	fifofullid;	/* broadcast FIFO interrupt id number */
	int	comm_areap;	/* Physical address of GM to host comm area */
	struct  cursorstuff_s cursorstuff;
	char	pad[752];	/* filler */
} gm2_shared_t;

/*
** Structure of data shared between the host and the GM1.
** This area contains state information that needs to be constantly
** available to both sides and needs to be updated quickly.
*/
typedef struct	gm1_shared_s {
	short	gmstate;	/* current state of the GM */
	short	cursorx;	/* current x value of the cursor */
	short	cursory;	/* current y value of the cursor */
	short	textPortReady;	/* TRUE if textPort is able to accept chars */
	short	pickready[16];	/* ready flag per context for pick/select */
	short	scrblank;	/* boolean: screen display blanked */
	short	longwordpad;	/* align the next field to a longword */
	int	rv_state;	/* rv, dg, vo, vp bits defined in gshm.h */
	struct  cursorstuff_s cursorstuff;
	char	pad[1984];	/* filler */
} gm1_shared_t;

/*
 * Data returned by VGX/SKYWRITER pipe for texture management.
 */
struct lamtable {
	long	texad;
	long	fogad;
};

typedef struct	gm_shared_s {
#if defined(IP5) || defined(IP17)
	int	gmstate;	/* current state of the GM */
	int	cursorx;	/* current x value of the cursor */
	int	cursory;	/* current y value of the cursor */
	int	textPortReady;	/* TRUE if textPort is able to accept chars */
	int	pickready[16];	/* ready flag per context for pick/select */
	int	scrblank;	/* boolean: screen display blanked */
	int	pipecpu;	/* id of cpu to interrupt (owner of pipe) */
	int	feedp[16];	/* ptrs to feedback areas (1 per context) */
	int	storep;		/* ptr to store context area */
	int	storelim;	/* size in bytes of store context area */
	int	fifofull;	/* GM to host fifo full flag */
	int	cputype;	/* cpu type (ip5 or ip7) */
	int	broadcast;	/* broadcast FIFO interrupt for share groups */
	int	rv_state;	/* rv, dg, vo, vp bits defined in gshm.h */
	gm_dispinfo_t dispinfo;	/* display information (described above) */
	int	fifofullid;	/* broadcast FIFO interrupt id number */
	int	comm_areap;	/* Physical address of GM to host comm area */
	struct  cursorstuff_s cursorstuff;
	int	lamready;	/* Ready flag for lamtable to be valid */
	struct  lamtable lamtable[16];	/* One lam entry for each HW context */
	int	little_endian[3];/* Set to 0xffffffff if host is little endian
				* and the interface to MP bus is also little endian.
				* Set to 0xf0f0f0f0 if host is little endian but
				* the bus interface is still big-endian.  Set
				* to all zeros if everything is big endian.
				*/
	int	vsync_cnt;	/* Initialized by GM via a pipe command, incremented by GM on every vert. retrace */
	char	pad[604];	/* filler - 16 for new stuff */
#else /* IP4 */
	short	gmstate;	/* current state of the GM */
	short	cursorx;	/* current x value of the cursor */
	short	cursory;	/* current y value of the cursor */
	short	textPortReady;	/* TRUE if textPort is able to accept chars */
	short	pickready[16];	/* ready flag per context for pick/select */
	short	scrblank;	/* boolean: screen display blanked */
	short	longwordpad;	/* align the next field to a longword */
	int	rv_state;	/* rv, dg, vo, vp bits defined in gshm.h */
	struct  cursorstuff_s cursorstuff;
	char	pad[1984];	/* filler */
#endif
} gm_shared_t;

/*
 * Values for gm_shared_t.gmstate
 */
#define	GMS_PROM	0xbaad	/* executing the GM prom code */
#define	GMS_DEAD	0xdead	/* executing GM prom code which failed diags. */
#define GMS_OS		0xbeef	/* running the GM operational sw */
#define	GMS_DIAG	0xcede	/* running some prog on top of PROM */

/*
|| Structure for low portion of GM DRAM
*/
typedef struct gm_dram_s {
#if defined(IP5) || defined(IP17)
	host_to_gm_comm_t host_to_gm_comm;
	gm_to_host_comm_t gm_to_host_comm;
	int diagstuff[256];
	gm_shared_t gm_shared_data;
	gm_shared_t gm_shared_data1;	/* for pipe 1*/
	int diagstuff0[256];		/* for pipe 1*/
#else /* IP4 */
	int evt[256];		/* Exception vector table */
	host_to_gm_comm_t host_to_gm_comm;
	gm_to_host_comm_t gm_to_host_comm;
	int diagstuff[256];
	gm_shared_t gm_shared_data;
	int unused[5];
#endif
} gm_dram_t;

/*
 * Status register values
 */
#define GM_TO_HOST_STATUS	0x2	/* GM2 to Host */
#define	FIFO_TO_HOST_STATUS	0x3	/* GM2 FIFO interrupt status */
#define	BROAD_TO_HOST_STATUS	0x4	/* GM2 Broadcat FIFO interrupt status */

#define GM3_TO_HOST_STATUS	0x5	/* GM3 to Host */

#define HOST_TO_GM_STATUS	0x0	/* Host to GM */

#if defined(IP5GT) || defined(IP7GT) || defined(SKYWR) || defined(IP17SKY) || defined(IP7) || defined(IP17)
/*
 * Descriptor for pixel buffer dma command, passed from host to GM
 * through the vertex buffer.
 */
struct chunk {
	int offset;	/* offset in bytes into the pixel buffer */
	char *addr;	/* physical MP bus address */
	int len;	/* length of transfer in bytes */
};

struct gm_pixel_dma_cmd {
	int nchunks;	/* number of discontiguous chunks in the host buffer */
	struct chunk chunks[3];	/* chunk descriptors for the host buffer */
};

/*
 * The vertex buffer contains 8 shorts destined for the pp,
 * followed an array of struct gm_pixbuf_desc's,
 * terminated by one with (srcheight == 0).
 *
 * Each of these describes the physical pages for up to one pixel-buffer's
 * worth of data (which may span up to 3 different pages).
 */

struct gm_pixbuf_desc {
	unsigned short	ypos;		/* starting screen space y position */
	unsigned short	maxydest;	/* ypos + destheight - 1 */
	struct chunk	chunks[3];	/* set len = 0 if absent */
};

#endif /* defined(IP5GT) || defined(IP7GT) || defined(SKYWR) || defined(IP17SKY) || defined(IP7) || defined(IP17) */
#endif /* !ASM68K */

/**************** Commands that the GM sends to the host ******************/

#define HOST_ABORT			0	/* aborted current req */
#define	HOST_CONTEXTSWITCHCOMPLETE	54
#define	HOST_SWAPBUFFERSCOMPLETE	55
#define	HOST_RESET_COMPLETE		56
#define	HOST_PUTCHAR			57
#define	HOST_ERROR			58
#define	HOST_ACKNOWLEDGE		59
#define	HOST_DIAG_CMD			60
#define	HOST_FEEDBACK			61
#define	HOST_STORE_CX			62
#define	HOST_PIXEL_DMA_DONE		63
#define	HOST_FIFO_FULL			64
#define	HOST_BROAD_FIFO_FULL		65
#define	HOST_COPY_DONE			66
#define HOST_GSYNCCOMPLETE		67
#define	HOST_LAMTABLE			68
#define	HOST_PIXDMA_DONE		69
#define HOST_VIDEO_PERIPH_ACK		70
#define HOST_VSYNC			71

#define GM_FIFO_SIZE		0x4000
#define GM_FEED_SIZE		0x3000 
#define GM_FEED_OFFSET		0x002000  
#define GM_UNLOAD_OFFSET	0x032000

#endif /* __SYS_GMHOST_H__ */
