#ifndef __SYS_MBUF_H__
#define __SYS_MBUF_H__

/* Copyright (C) 1988,1989,1990 Silicon Graphics, Inc. All rights reserved. */
/*
 * Copyright (c) 1982, 1986, 1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of California at Berkeley. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 *
 *	@(#)mbuf.h	7.10 (Berkeley) 2/8/88
 */

/*
 * Constants related to memory allocator.
 */
#define	MSIZE	    128			/* size of an mbuf */

#define VCL_MAX	    CLBYTES		/* largest variable cluster */
#define	MCLBYTES    CLBYTES

#define	MMINOFF	    16			/* mbuf header length */
#define	MTAIL	    0
#define	MMAXOFF	    (MSIZE-MTAIL)	/* offset where data ends */
#define	MLEN	    (MSIZE-MMINOFF-MTAIL)   /* mbuf data length */
#define	NMBCLUSTERS 1024		/* nominal max # of clusters */
#define	NMBPCL	    (CLBYTES/MSIZE)	/* # mbufs per cluster */

/*
 * Macros for type conversion
 */

/* address in mbuf to mbuf head */
#define	datatom(x)	((struct mbuf *)((int)(x)-1 & ~(MSIZE-1)))
#define sotom(x)	datatom(x)
#define rttom(x)	datatom(x)
#define rcbtom(x)	datatom(x)
#define inpcbtom(x)	datatom(x)
#define tcptmplttom(x)	datatom(x)
#define unptom(x)	datatom(x)
#define mholdtom(x)	datatom(x)

/* mbuf head, to typed data */
#define	mtod(x,t)	((t)((int)(x) + (x)->m_off))

struct mbuf {
	struct	mbuf *m_next;		/* next buffer in chain */
	u_long	m_off;			/* offset of data */
	short	m_len;			/* amount of data in this mbuf */
	u_char	m_flags;		/* attributes */
	u_char	m_type;			/* mbuf type (MT_FREE == free) */
	struct	mbuf *m_act;		/* link in higher-level mbuf list */
	union {
		/*
		 * On SGI systems, clusters can be special.  Special
		 * clusters have their underlying memory somewhere else.
		 * The freefunc is called with the given arguments when the
		 * buffer is done being used.
		 */
		struct {
			int	(*mu_freefunc)();
			long	mu_farg;
			int	(*mu_dupfunc)();
			long	mu_darg;
		} m_us;
		u_char	mu_dat[MLEN];	/* data storage */
	} m_u;
};
#define	m_dat		m_u.mu_dat		/* compatability */
#define	m_freefunc	m_u.m_us.mu_freefunc	/* lazy */
#define	m_farg		m_u.m_us.mu_farg
#define	m_dupfunc	m_u.m_us.mu_dupfunc
#define	m_darg		m_u.m_us.mu_darg
#define	m_cluster	m_flags			/* compatibility */

/* mbuf types */
#define	MT_FREE		0	/* should be on free list */
#define	MT_DATA		1	/* dynamic (data) allocation */
#define	MT_HEADER	2	/* packet header */
#define	MT_SOCKET	3	/* socket structure */
#define	MT_PCB		4	/* protocol control block */
#define	MT_RTABLE	5	/* routing tables */
#define	MT_HTABLE	6	/* IMP host tables */
#define	MT_ATABLE	7	/* address resolution tables */
#define	MT_SONAME	8	/* socket name */
#define	MT_SOOPTS	10	/* socket options */
#define	MT_FTABLE	11	/* fragment reassembly header */
#define	MT_RIGHTS	12	/* access rights */
#define	MT_IFADDR	13	/* interface address */
#define	MT_DN_DRBUF	14	/* 4DDN driver buffer allocation */
#define	MT_DN_BLK	15	/* 4DDN block allocation */
#define	MT_DN_BD	16	/* 4DDN buffer to mbuf */
#define MT_IPMOPTS	17	/* internet multicast options */
#define MT_MRTABLE	18	/* multicast routing tables */
#ifdef XTP
#define	MT_X_CTRL	19	/* xtp_sendcontrol packet */
#define	MT_X_DIAG	20	/* xtp_senddiag packet */
#define	MT_X_MKPKT	21	/* xtp_outqcopy header/data */
#define	MT_X_MKPKT_T	22	/* xtp_outqcopy trailer */
#define	MT_X_OQ		23	/* xtp_outqappend data */
#define	MT_X_CTXT	24	/* newcontext */
#define	MT_X_NAME	25	/* xsd_matchaddr accept */
#define MT_MAX		26
#else
#define MT_MAX		19	/* 'largest' type */
#endif

/* mbuf flags attributes */
#define MCL_CLUSTER	0x01	/* undistinguished cluster */
#define M_CKSUMMED	0x02	/* link-level handles checksums */
#define M_BCAST		0x40	/* send/received as link-level broadcast */
#define M_MCAST		0x80	/* send/received as link-level multicast */
#define M_COPYFLAGS	(M_CKSUMMED|M_MCAST|M_BCAST)

/* flags to m_get */
#define	M_DONTWAIT	0
#define	M_WAIT		1


/* length to m_copy to copy all */
#define	M_COPYALL	1000000000

/*
 * m_pullup() will pull up additional length if convenient;
 * should be enough to hold headers of second-level and higher protocols.
 */
#define	MPULL_EXTRA	32


/* 4.3BSD compatiblity macros
 */
#define	MFREE(m, n)	((n) = m_free(m))
#define	MGET(m, i, t)	((m) = m_get(i, t))
#define	MGETHDR(m, i, t) ((m) = m_get(i, t))
#define	M_HASCL(m)	(0 != ((m)->m_flags & MCL_CLUSTER))

/* this is similar to MCLALLOC() and MCLGET(), allocating a buffer of
 *	CLBYTES.  Unlike the 4.3BSD macros, it does allocates the little
 *	mbuf as well as the page.
 */
#define MCL_GET(m, i)	(m = m_vget(M_DONTWAIT,CLBYTES,MT_DATA))

/* this releases the CLBYTES mbuf allocated byte MCL_GET() */
#define MCL_FREE(m)	m_freem(m)

/* there are no equivalents to MTOCL(), mtocl(), and cltom() */


/*
 * Mbuf statistics.
 */
struct mbstat {
	u_long	m_mbufs;	/* mbufs obtained from page pool */
	u_long	m_clusters;	/* clusters obtained from page pool */
	u_long	m_spare;	/* spare field */
	u_long	m_clfree;	/* free clusters */
	u_long	m_drops;	/* times failed to find space */
	u_long	m_wait;		/* times waited for space */
	u_long	m_drain;	/* times drained protocols for space */
	u_short	m_mtypes[MT_MAX];	/* type specific mbuf allocations */
};

#ifdef	_KERNEL
extern struct mbstat mbstat;

extern struct mbuf *m_get(int, int);	/* alloc an mbuf */
extern struct mbuf *m_getclr(int, int);	/* alloc and zero-fill an mbuf */
extern struct mbuf *m_vget(int, int, int);  /* alloc variable-sized cluster */
extern struct mbuf *m_cvget(int, int, int*);	/* alloc cache-clean */
extern void	    m_flush(struct mbuf*,void*, int);    /* dcache_clean */
extern struct mbuf *m_free(struct mbuf*);   /* free an mbuf/cluster */
extern void	    m_freem(struct mbuf*);  /* free a chain */
extern struct mbuf *m_copy(struct mbuf*, int, int);
extern int  m_datacopy(struct mbuf*, int, int, caddr_t);    /* mbuf to buf */
extern int  m_flip(struct mbuf*, caddr_t);  /* flip to user */
extern void m_cat(struct mbuf*, struct mbuf *);	/* concatenate */
extern void m_adj(struct mbuf*, int);	/* trim an mbuf */
extern struct mbuf *m_pullup(struct mbuf*, int);    /* condense an mbuf */
extern int	   m_length(struct mbuf*);  /* data length of chain */
#endif /* _KERNEL */

#endif /* !__SYS_MBUF_H__ */
