/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef __SYS_MOUNT_H__
#define __SYS_MOUNT_H__
#ident	"$Revision: 3.15 $"

#include "sys/sema.h"

/*
 *	Mount structure.
 *	One allocated on every mount.
 *
 * Locking:
 *	A mount table lock (mnt_lck) single threads the mounting and unmounting
 *	process.
 *
 *	A mount table entry lock (mp->m_sema) prevents a race condition with
 *	the unmount process and path name lookup.
 *
 *	Protocol:
 *		mounted-on inode
 *		THEN mount table lock
 *		[THEN mount table entry lock]
 *		THEN root inode
 * mounting:
 *
 * unmounting:
 *	inodes - while trying to flush inodes, hold ifreelist lock
 *	once unmount cannot fail, set mflags to MOFFLINE and release ifreelist
 *	iget checks if it had to sleep if file system went free/offline
 *	alloc/sb - shouldn't happen since MUST have inode locked in order
 *	to get into any routines that would call alloc/free/etc
 *	thus iflush would fail
 *
 * Mount setup:
 *	mounted on inode		root-inode for new fs
 * 	(in parents fs)
 *
 *	ip<---------------------------|    |-->ip
 *	i_mnton------|		      |    |
 *		     |   	      |    |
 *	i_mntdev-|   |		      |    |
 *		 |   |		      |    |
 *	 	 |   |->mp (new fs)   |    |
 *	 	 |	m_inodp-------|    |
 *		 |			   |
 *	  	 |	m_mount------------|
 *		 |
 *		 |----->mp (parent fs)
 *       IMOUNT set                            IISROOT set
 */

struct	mount {
	ushort		m_flags;	/* status */
	short		m_fstyp;	/* file system type */
	dev_t		m_dev;		/* device mounted */
	caddr_t		m_bufp;		/* fs-dependent mount data */
	struct inode	*m_inodp;	/* mounted-on inode */
	struct inode	*m_mount;	/* root inode */
	long		m_bcount;
	sema_t		m_sema;		/* mount table entry lock */
	struct inode	*m_qinod;	/* pointer to quota file */
	u_long		m_btimelimit;	/* block time limit */
	u_long		m_ftimelimit;	/* file time limit */
	u_long		m_qindex;	/* index size of quotas file */
};

/* allocation flags (m_flags) */

#define	MFREE		0x0	/* Unused - free */
#define	MINUSE		0x1	/* In use - fs mounted on this entry */
#define	MINTER		0x2	/* Mount system call is using this entry */
#define	MRDONLY		0x4	/* File system is mounted read only */
#define	MEXPORTED	0x8	/* File system is exported (NFS) */
#define	MOFFLINE	0x10	/* file system is going offline - set ONLY */
				/* after umount can't fail */
#define	MGRPID		0x200	/* BSD group-id on create */
#define MNOSUID		0x400	/* mount fs with setuid not allowed */
#define MQ_ENABLED	0x800	/* quotas are enabled */
#define MQ_ACTIVE	0x1000	/* quotas being changed, but not enabled */
#define MNODEV		0x2000	/* disallow access to device files */

#define	rdonlyfs(mp)	((mp)->m_flags & MRDONLY)

/* Flag bits passed to the mount system call */
#define	MS_RDONLY	0x1	/* read only bit */
#define	MS_FSS		0x2	/* Old (4-argument) mount (compatibility) */
#define	MS_DATA		0x4	/* 6-argument mount */
#define	MS_GRPID	0x8	/* BSD group-id on create */
#define MS_NOSUID	0x10	/* no setuid */
#define MS_NODEV	0x20	/* disallow access to device files */

#ifdef _KERNEL
extern struct mount mount[];
extern sema_t mnt_lck;		/* sema lock for entire mount table */
extern int devnotinuse(dev_t, struct inode *, int);
#endif

#endif	/* __SYS_MOUNT_H__ */
