/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef __SYS_PARAM_H__
#define	__SYS_PARAM_H__
#pragma once
#ident	"$Revision: 3.35 $"
/*
 * fundamental variables
 * don't change too often
 */

#define	MAXPID	30000		/* max process id */
#define	MAXUID	60000		/* max user id */
#define	MAXLINK	30000		/* max links */

#define	SSIZE	1		/* initial stack size (*NBPP bytes) */
#define	SINCR	1		/* increment of stack (*NBPP bytes) */
#define	USIZE	2		/* size of user block (*NBPP bytes) */

#define	CANBSIZ	256		/* max size of typewriter line	*/
#define	HZ	100		/* 100 ticks/second of the clock */

#define	NCARGS	10240		/* # characters in exec arglist */
				/*   must be multiple of NBPW.  */
				/* this is minimum value - tunable */

/*	The following define is here for temporary compatibility
**	and should be removed in the next release.  It gives a
**	value for the maximum number of open files per process.
**	However, this value is no longer a constant.  It is a
**	configurable parameter, NOFILES, specified in the kernel
**	master file and available in v.v_nofiles.  Programs which
**	include this header file and use the following value may
**	not operate correctly if the system has been configured
**	to a different value.
*/

#define	NOFILE	20

/*	The following represent the minimum and maximum values to
**	which the paramater NOFILES in the kernel master file may
**	be set.
*/

#define	NOFILES_MIN	 20
#define	NOFILES_MAX	200

/*	The following defines apply to the kernel virtual
**	address space.  This number must be big enough to
**	hold the buffer cache, kernel dynamic memory and other
**	smaller users.
*/

/*
 * priorities
 * should not be altered too much
 */

#define	PMASK	0177
#define	PCATCH	0400
#define	PRECALC	01000
#define	PNOSTOP	02000
#define	PSWP	0
#define	PINOD	10
#define	PRIBIO	20
#define	PZERO	25
#define PMEM	0
#define	NZERO	20
#define	PPIPE	26
#define	PWAIT	29
#define	PSLEP	39
#define	PUSER	60
#define	PIDLE	255

/*
 * fundamental constants of the implementation--
 * cannot be changed easily
 *
 * On the MIPS CPU a click is a page (4K), a segment is what a click can
 * map (i.e. a page of ptes or 4K/8 * 4K = 2 Meg)
 */

#define NBPS	(NCPS*NBPC)	/* Number of bytes per segment */
#define	NBPW	sizeof(int)	/* number of bytes in an integer */
#define	NCPS	(NBPC/8)	/* Number of clicks per segment */
#define CPSSHIFT	9	/* LOG2(NCPS) if exact */
#define	NBPC	4096		/* Number of bytes per click */
#define	BPCSHIFT	12	/* LOG2(NBPC) if exact */
#ifndef NULL
#define	NULL	0
#endif
#define	CMASK	022		/* default mask for file creation */
#define	CDLIMIT	0x7fffffff	/* default max write address */
#define	NODEV	(dev_t)(-1)

/*
 * XXX These should be expunged, and BBSHIFT should be defined as 9.
 */
#define	NBPSCTR		512	/* Bytes per disk sector.	*/
#define SCTRSHFT	9	/* Shift for BPSECT.		*/

/*
 * Block I/O parameterization.  A basic block (BB) is the lowest size of
 * filesystem allocation, and must == NBPSCTR.  Length units given to bio
 * routines are in BB's.
 */
#define	BBSHIFT		9
#define	BBSIZE		(1<<BBSHIFT)
#define	BBMASK		(BBSIZE-1)
#define	BTOBB(bytes)	(((unsigned long)(bytes) + BBSIZE - 1) >> BBSHIFT)
#define	BTOBBT(bytes)	((unsigned long)(bytes) >> BBSHIFT)
#define	BBTOB(bbs)	((bbs) << BBSHIFT)

#define SEEKLIMIT	0x7fffffff
#define BBSEEKLIMIT	BTOBBT(SEEKLIMIT)

#define	UMODE	3		/* current Xlevel == user */
/* in mips the psw is the status register */
#ifdef R3000
#define	USERMODE(psw)	(((psw) & SR_KUP) == SR_KUP)
#endif
#ifdef R4000
#define	USERMODE(psw)	(((psw) & SR_KSU_MSK) == SR_KSU_USR)
#endif
#define	BASEPRI(psw)	(((psw) & SR_IMASK) == SR_IMASK)

#ifdef MIPSEB
#define	lobyte(X)	(((unsigned char *)&X)[1])
#define	hibyte(X)	(((unsigned char *)&X)[0])
#define	loword(X)	(((ushort *)&X)[1])
#define	hiword(X)	(((ushort *)&X)[0])
#else
#define	lobyte(X)	(((unsigned char *)&X)[0])
#define	hibyte(X)	(((unsigned char *)&X)[1])
#define	loword(X)	(((ushort *)&X)[0])
#define	hiword(X)	(((ushort *)&X)[1])
#endif

#ifdef _KERNEL
/*
 * DELAY(n) should be n microseconds, roughly. 
 */
#define DELAY(n)	us_delay(n)
/* to guarentee delay between writes, use DELAYBUS */
#define DELAYBUS(n)	us_delaybus(n)

#define	MAXSUSE	255

/* timeout call with this tick results in immediate timepoke() 
 */
#define TIMEPOKE_NOW	-100
#endif /* _KERNEL */

/* REMOTE -- whether machine is primary, secondary, or regular */
#define SYSNAME 9		/* # chars in system name */
#define PREMOTE 39

/*
 * Maximum size of hostname and domainname recognized and stored in the
 * kernel by sethostname and setdomainname
 */
#define MAXHOSTNAMELEN 64

/* POSIX version number, returned by sysconf() system call */
#ifndef _POSIX_VERSION
#define _POSIX_VERSION	198808L
#endif


/*
 * The remainder of this file is for BSD compatibility.
 */
#include "sys/signal.h"
#include "sys/types.h"

/*
 * NOTE: MAXPATHLEN should be kept in sync with PATH_MAX, in limits.h.
 */
#define	MAXPATHLEN	1024	/* maximum pathname length */

/*
 * The following two defines are for BSD-like multiple groups.  Note that
 * the actual configuration value is in master.d/kernel.  The #define there
 * is NGROUPS_MAX, the corresponding integer is ngroups_max (ala SVR4).
 */
#define NGROUPS_UMAX	32	/* for SVR4 compatibility */
#define NGROUPS_UMIN	8	/*	  "	*/
#define _NOGROUP	65535	/* indicates end of group list */
#ifdef _KERNEL
extern int ngroups_max;		/* master.d/kernel */
#endif /* _KERNEL */

/* 
 * NGROUPS must not be set greater than NGROUPS_MAX in master.d/kernel.
 * Compilation will not succeed in that case.
 */
#define NGROUPS		16	/* max # groups process may be in */
#define NOGROUP	65535		/* indicates end of group list */

/*
 * Macros for fast min/max.
 */
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

/*
 * Macros for counting and rounding.
 */
#ifndef howmany
#define	howmany(x, y)	(((x)+((y)-1))/(y))
#endif
#define	roundup(x, y)	((((x)+((y)-1))/(y))*(y))

#endif	/* __SYS_PARAM_H__ */
