/*
 * IP6/IP12/IP20 Parallel Port
 *
 * $Revision: 1.11 $
 *
 */
#define PLPIOC	('p' << 8)

#define PLPIOCSTATUS	(PLPIOC|1)	/* returns status byte defined below */
#define PLPIOCRESET	(PLPIOC|2)	/* performs printer reset */
#define PLPIOCSETMASK	(PLPIOC|3)	/* set fatal status mask */
#define	PLPIOCTYPE 	(PLPIOC|4)	/* set versatec/centronics */
#define	PLPIOCSTROBE 	(PLPIOC|5)	/* set strobe length/duty cycle */
#define	PLPIOCIGNACK 	(PLPIOC|6)	/* ignore ack input */
#define	PLPIOCWTO 	(PLPIOC|7)	/* set write timeout in secs */
#define	PLPIOCRTO 	(PLPIOC|8)	/* set read timeout in secs */
#define	PLPIOCREAD 	(PLPIOC|9)	/* enable reading from port */

/* status bits */
#define PLPFAULT	1	/* printer fault */
#define PLPEOP		2	/* end-of-paper condition */ 
#define PLPEOI		4	/* end-of-ink (or ribbon out) */
#define PLPONLINE	8	/* printer is on-line */

/* printer types */
#define PLP_TYPE_VERS	1	/* inverts STB and ACK signals */
#define PLP_TYPE_CENT	2

/* strobe length 
 *
 * 	dc = total time for strobe in 30 ns ticks (dc < 127)
 * 	fall = time until strobe falls in 30 ns ticks (time < dc)
 * 	rise = time until strobe rises again in 30 ns ticks (rise < dc)
 *
 *	---------------|             |---------------
 *		       |             |
 *                     |-------------|
 *      <---  fall --->
 *                     <---  rise --->
 *      <------------------  dc -------------------->
 */
#define PLP_STROBE(dc,fall,rise)	\
	(((dc&0x7f)<<24)|(((dc-fall)&0x7f)<<16)|(((dc-fall-rise)&0x7f)<<8))

