#ifndef __SYS_RRM_H__
#define __SYS_RRM_H__
/**************************************************************************
 *		 Copyright (C) 1990, Silicon Graphics, Inc.		  *
 *  These coded instructions, statements, and computer programs	 contain  *
 *  unpublished	 proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may	not be disclosed  *
 *  to	third  parties	or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 **************************************************************************/

/*
 *   $Revision: 1.41 $
 */


/*
 * Device independent rendering resources that can be requested
 */
#define	RRM_RMASK_PCX		0x00000001	/* graphics context */
#define	RRM_RMASK_CLIP		0x00000002	/* clipping region */
#define	RRM_RMASK_BUFFER	0x00000004	/* buffer (double buffering) */
#define	RRM_RMASK_SWAPBUF	0x00000008	/* ability to swap buffers */

#define	RRM_RMASK_RETRACE	0x00000010	/* retrace event occurred  */
#define	RRM_RMASK_MESSAGE	0x00000020	/* messaged manager */
#define	RRM_RMASK_ALL		(RRM_RMASK_PCX 	    | \
				 RRM_RMASK_CLIP     | \
				 RRM_RMASK_BUFFER   | \
				 RRM_RMASK_SWAPBUF  | \
				 RRM_RMASK_RETRACE  | \
				 RRM_RMASK_MESSAGE)

/*
 * rendering resource manager operations
 *
 * WARNING!  These RRM command numbers are used as an index into the
 * RRMfncs table so if they change, so must the RRMfncs table.
 */
#define RRM_BASE	      1000
#define RRM_OPENRN	      (RRM_BASE+0)  /* open rendering node */
#define RRM_CLOSERN	      (RRM_BASE+1)  /* close rendering node */
#define RRM_BINDPROCTORN      (RRM_BASE+2)  /* set current RN for proc */
#define RRM_BINDRNTOCLIP      (RRM_BASE+3)  /* bind RN to given clip region */
#define RRM_UNBINDRNFROMCLIP  (RRM_BASE+4)  /* unbind RN from clip region */
#define RRM_SWAPBUF	      (RRM_BASE+5)  /* swap buffers */
#define RRM_SETSWAPINTERVAL   (RRM_BASE+6)  /* swapinterval */
#define RRM_WAITFORRETRACE    (RRM_BASE+7)  /* wait for retrace */
#define RRM_SETDISPLAYMODE    (RRM_BASE+8)  /* display mode */
#define RRM_MESSAGE	      (RRM_BASE+9)  /* send message to board mgr */
#define RRM_INVALIDATERN      (RRM_BASE+10) /* invalidate resources in a RN */
#define RRM_VALIDATECLIP      (RRM_BASE+11) /* validate a RN's clip resource */
#define RRM_VALIDATESWAPBUF   (RRM_BASE+12) /* validate swap buffer ability */
			      /* XXX 13 available */
#define RRM_VALIDATEBUFFER    (RRM_BASE+14) /* validate current buffer */
#define RRM_VALIDATEMESSAGE   (RRM_BASE+15) /* ack message */
#define RRM_GETDISPLAYMODES   (RRM_BASE+16) /* get display mode registers */
#define RRM_LOADDISPLAYMODE   (RRM_BASE+17) /* display mode */
#define RRM_CMD_LIMIT	      (RRM_BASE+100)

/*
 * argument structures
 *
 * WARNING!  These structs MUST have rnid as their first argument.
 */

struct RRM_OpenRN {
	long		rnid;		/* rendering node ID */
	unsigned long	rmask;		/* resource mask: ones needed */
};

struct RRM_CloseRN {
	long		rnid;		/* rendering node ID */
};

struct RRM_BindProcToRN {
	long		rnid;		/* rendering node ID */
};

struct RRM_BindRNToClip {
	long		rnid;		/* rendering node ID */
	long		clipid;		/* opaque clip region ID */
};

struct RRM_UnbindRNFromClip {
	long		rnid;		/* rendering node ID */
};

struct RRM_SwapBuf {
	long		rnid;		/* rendering node ID */
	long		hwd_mode;	/* hardware dependent register value */
	long		hwi_mode;	/* hardware independent disp mode description (M_) */
};

struct RRM_SetSwapInterval {
	long		rnid;		/* rendering node ID */
	long		swapinterval;	/* number of frames between swaps */
};

struct RRM_WaitForRetrace {
	long		rnid;		/* rendering node ID */
};

struct RRM_SetDisplayMode {
	long		rnid;		/* rendering node ID */
	long		hwd_mode;	/* hardware dependent register value */
	long		hwi_mode;	/* hardware independent mode description (M_) */
};

struct RRM_LoadDisplayMode {
	long		rnid;		/* rendering node ID */
	long		wid;		/* window ID that should be used */
	long		hwd_mode;	/* hardware dependent register value */
	long		hwi_mode;	/* hardware independent mode description (M_) */
};

struct RRM_GetDisplayModes {
	long		rnid;		/* rendering node ID */
	long		*buf;		/* buffer for mode descriptions (M_) */
	unsigned long	len;		/* length of buffer passed */
};

struct RRM_InvalidateRN {
	long		rnid;		/* rendering node ID */
	unsigned long	rmask;		/* resource mask: ones to invalidate */
	unsigned long	hwi_mode;	/* return value - mode window wants */
	unsigned long	flags;		/* input/output flags */
};
/* flags for RRM_InvalidateRN.flags */
#define	RRM_INVAL_WRONGBUF	0x1	/* return: DID left in wrong buffer */

struct RRM_PieceList {			/* window clipping piece list piece */
	long		x;		/* lower left x */
	long		y;		/* lower left y */
	long		xsize;		/* size in x */
	long		ysize;		/* size in y */
};

/* These arguments are interpreted by the dev. dependent graphics drivers.
   All arguments are not used by all drivers.
   We can probably simplify this interface. */
struct RRM_ValidateClip {
	long		rnid;		/* rendering node ID */
	long		xorg;		/* window x origin */
	long		yorg;		/* window y origin */
	long		xsize;		/* window x size */
	long		ysize;		/* window y size */
	long		numpieces;	/* number pieces in clip region */
	struct RRM_PieceList *piecelist;/* window clipping piece list:
					   can be NULL (e.g. numpieces=1) */
	long		wid;		/* window ID that should be used */
	long		obscured;	/* window obscured =1, else =0 */
	long		widcheck;	/* WID check enabled =1, else =0 */
	long		changed;	/* window moved =1, shrunk =2, else =0*/
};

struct RRM_ValidateSwapBuf {
	long		rnid;		/* rendering node ID */
	long		swapbufid;	/* opaque swap buffers ID */
};

struct RRM_ValidateBuffer {
	long		rnid;		/* rendering node ID */
};

struct RRM_ValidateMessage {
	long		rnid;		/* rendering node ID */
};


/*
 * Messege types being sent to board manager.
 */
#define	RRM_MSG_BINDRNTOCLIP	1	/* bind given RN to given clip region */
#define	RRM_MSG_UNBINDRNFROMCLIP 2	/* unbind given RN from clip region */
#define	RRM_MSG_VALIDATECLIP	3	/* validate a given clip region */
#define	RRM_MSG_VALIDATESWAPBUF	4	/* validate ability to swap buffers */
#define	RRM_MSG_SETDISPLAYMODE	5	/* set display mode */

	/*  these are generated by a GL program */
#define	RRM_MSG_FULLSCREEN	6	/* turn clipping off/on */
#define	RRM_MSG_ZBUFFERED	7	/* notify zbuffering */
#define	RRM_MSG_DONTFREEZE	8	/* dont freeze proc when win unmapped */


#define GFX_MESSAGE_MAX_SIZE 10

struct RRM_Message_Header {
	long		rnid;		/* rendering node ID */
	unsigned long	rmask;		/* resource mask: ones to invalidate */
	unsigned char   type;		/* message type */
	long 		length;		/* message length */
};

struct RRM_Message_Fullscreen {
	struct RRM_Message_Header header;
	long clipid;
	long rnid;
	unsigned long flag;
};

struct RRM_Message_Dontfreeze {
	struct RRM_Message_Header header;
	long clipid;
	long rnid;
	unsigned long flag;
};

#ifdef _KERNEL

struct RRM_Message {
	struct RRM_Message_Header header;
	unsigned long data[GFX_MESSAGE_MAX_SIZE]; 
};
	
/*
 *   structure of an rrm message going upstream
 */

struct gfx_rrm_message {
	struct gfx_board *board;
	unsigned char type;
	long size;
	unsigned long int arg[GFX_MESSAGE_MAX_SIZE];
};

/*
 * Device INDEPENDENT rendering node structure
 */
struct rrm_rnode {
	long		gr_free;	/* RN is free if 1, allocated if 0 */
	union {
		long	gr_rnid;
		struct {
			short incarn;
			short rnid;
		} rnid;
	} gr_rnid;
	unsigned long	gr_reqmask;	/* rendering resource request mask */
	unsigned long	gr_validmask;	/* rendering resource valid mask */
	void		*gr_pcx;	/* pipe context pointer */
	long		gr_pcxsize;	/* pipe context size (in bytes) */
	long		gr_clipid;	/* opaque clip region ID */
	long		gr_swapbufid;	/* opaque swap buffers ID */
	long		gr_swapinterval;/* swap interval for this RN */
	ulong		gr_nextswaptime;/* last swap time for this RN */
	unsigned long	gr_hwi_mode;	/* opaque HW indep. display mode */
	struct rrm_rnode *gr_vrwaitnext;/* next RN waiting for retrace */
	sema_t		gr_rnwait;	/* sleep on this RN */
	struct gfx_gfx	*gr_gfxp;	/* gfx struct using this RN */
	long		gr_pcxid;	/* context ID on rendering node */
	struct rrm_rnode *gr_older;	/* next older rnp (pcxid allocation) */
	struct rrm_rnode *gr_newer;	/* next newer rnp (pcxid allocation) */
	lock_t		gr_rnlock;	/* lock for this RN (MP) */
	void		*gr_private;	/* pointer to device dependent stuff */
};

/*
 * Element of a linked list of pointers to rendering nodes
 */
struct rrm_link {
	struct rrm_rnode	*gl_rnp;
	struct rrm_link		*gl_next;
};

#endif /* _KERNEL */

#endif /* __SYS_RRM_H__ */
