/**************************************************************************
 *									  *
 * 		 Copyright (C) 1990, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ifndef __SYS_SYSMP_H__
#define __SYS_SYSMP_H__

#ifdef __cplusplus
extern "C" {
#endif

#ident "$Revision: 3.31 $"

/*
 * sysmp.h - defined/structs for sysmp() - multi-processor specific calls
 *
 * Format:
 *	sysmp(opcode, [opcode_specific_args])
 */

/* possible opcodes */
#define MP_NPROCS	1	/* # processor in complex */
#define MP_NAPROCS	2	/* # active processors in complex */
#define MP_ENABLE	4	/* enable a processor OBSOLETE */
#define MP_DISABLE	5	/* disable a processor OBSOLETE */
#define MP_KERNADDR	8	/* get address of kernel structure */
#define MP_SASZ		9
#define MP_SAGET	10
#define MP_SCHED	13	/* scheduler control calls */
#define MP_PGSIZE	14	/* get system page size */
#define MP_SAGET1	15	/* get system activity for a single CPU */
#define MP_EMPOWER	16	/* allow processor to sched any process */
#define MP_RESTRICT	17	/* restrict processor to mustrun processes */
#define MP_CLOCK	18	/* processor becomes (software) clock handler */
#define MP_MUSTRUN	19	/* assign process to this processor */
#define MP_RUNANYWHERE	20	/* let process run on any processor (default) */
#define MP_STAT		21	/* return processor status */
#define MP_ISOLATE	22	/* isolate a processor from inter-cpu intr */
#define MP_UNISOLATE	23	/* un-isolate a processor */
#define MP_PREEMPTIVE	24	/* turn clock back on an isolated processor */
#define MP_NONPREEMPTIVE	25	/* turn clock off of an isolated processor */

/* kernel address sub-opcodes */
#define MPKA_PROC		1	/* process table */
#define MPKA_VAR		2	/* kernel vars */
#define MPKA_SWPLO		3	/* swap lo */
#define MPKA_INO		4	/* inode table */
#define MPKA_FLE		5	/* file table */
#define	MPKA_NSPLOCK		6	/* number of kernel spin locks */
#define	MPKA_SEMAMETER		7	/* spin lock metering flag */
#define MPKA_SEMAPHORE		8	/* semaphore list */
#define MPKA_PROCSIZE		9	/* size of proc struct */
#define MPKA_TIME		10	/* time */
#define MPKA_MSG		11	/* msgque */
#define MPKA_SEM		12	/* sema */
#define MPKA_SHM		13	/* shmem */
#define MPKA_MSGINFO		14	/* msginfo */
#define MPKA_SEMINFO		15	/* seminfo */
#define MPKA_SHMINFO		16	/* shminfo */
#define	MPKA_SPLOCKMETER	17	/* spin lock metering flag */
#define	MPKA_SPLOCKMETERTAB	18	/* k_splockmeter table */
#define MPKA_AVENRUN		19	/* load average */
#define MPKA_PHYSMEM		20	/* &physmem */
#define MPKA_KPBASE		21	/* &kpbase */
#define MPKA_PFDAT		22	/* &kpbase */
#define MPKA_FREEMEM		23	/* &freemem */
#define MPKA_USERMEM		24	/* &usermem */
#define MPKA_PDWRIMEM		25	/* &pdwrimem */
#define MPKA_BUFMEM		26	/* &bufmem */
#define MPKA_BUF		27	/* buf */

/* system activities sub-opcodes */
#define MPSA_SINFO	1	/* get added-up sysinfo */
#define MPSA_MINFO	2	/* get minfo */
#define MPSA_DINFO	3	/* get dinfo */
#define MPSA_SERR	4	/* get syserr */
#define MPSA_PMTR	5	/* get pnmeter */
#define MPSA_NCMTR	7	/* get ncmeter */
#define MPSA_RMINFO	8	/* real memory info */
#define MPSA_BUFINFO	9	/* buffer cache info */

/* scheduler control call sub-opcodes */
#define MPTS_RTPRI	1	/* give a process a real-time priority */
#define MPTS_SLICE	2	/* alter a process time slice */
#define MPTS_RENICE	3	/* alter the nice value */

/* getpriority and setpriority emulation */
#define MPTS_RENICE_PROC 4	/* alter nice value of a process */
#define MPTS_RENICE_PGRP 5	/* alter nice value of a process group */
#define MPTS_RENICE_USER 6	/* alter nice value of a user */
#define MPTS_GTNICE_PROC 7	/* get nice value of process */
#define MPTS_GTNICE_PGRP 8	/* get nice value of process group */
#define MPTS_GTNICE_USER 9	/* get nice value of user */

/* More scheduler control call subcodes */
#define MPTS_SETHINTS	12	/* create a user scheduling hints structure */
#define MPTS_SCHEDMODE	13	/* set scheduling mode */
#define MPTS_SETMASTER	14	/* set master proc in a share group */

#if defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)
/* struct for real memory info */
struct rminfo {
	long		freemem;	/* current pages free memory */
	long		availsmem;	/* available real and swap memory */
	long		availrmem;	/* available real memory */
	long		bufmem;		/* attached to metadata cache */
	long		physmem;	/* total real memory */
	long		delwri;		/* delwri pages */
};

#ifndef _KERNEL
extern int sysmp(int, ...);
#endif /* !_KERNEL */
#endif

#ifdef __cplusplus
}
#endif

#endif /* __SYS_SYSMP_H__ */
