/**************************************************************************
 *									  *
 * 		 Copyright (C) 1989, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/
/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef __SYS_USER_H__
#define __SYS_USER_H__
#ident	"$Revision: 3.49 $"

#include "sys/types.h"
#include "sys/param.h"		/* NGROUPS_UMAX */
#include "sys/immu.h"		/* UADDR, MAXPSEGS */
#include "sys/pcb.h"		/* pcb_t */
#include "sys/reg.h"		/* EF_ELEM */
#include "sys/resource.h"	/* RLIM_NLIMITS */
#include "sys/profil.h"		/* struct prof */
#include "sys/sbd.h"		/* NWIREDENTRIES etc. */
#include "sys/signal.h"		/* NUMSIGS */
#include "sys/time.h"		/* struct itimerval */
#include "sys/syscall.h"	/* sysset_t */
#include "sys/fault.h"		/* fltset_t */

/*
 * The user structure.
 * One allocated per process.
 * Contains all per process data that doesn't need to be referenced
 * while the process is swapped.
 * The user block is USIZE*click bytes long;
 * resides at virtual kernel UADDR (defined in sbd.h);
 * is cross referenced with the proc structure for the same process.
 */
 
#define	PSARGSZ		80	/* Space in u-block for exec arguments.	*/
				/* Used by ps command.			*/

#define	PSCOMSIZ	32	/* Space in u-block for program being execed */

typedef	struct	user
{
	pcb_t	u_pcb;			/* MUST be first */
	uint	u_eframe[EF_ELEM];	/* save exception state */
	caddr_t	u_depth;		/* allow nesting */

	int 	(*u_sigtramp)();/* address of user's signal return */
				/* handler: _sigtramp in libc	*/

	void	(*u_signal[NUMSIGS])();	/* disposition of signals */
	ulong	u_sigmasks[NUMSIGS];	/* block-masks during handlers */
					/* (for POSIX and BSD signals) */
	int	u_sigonstack;		/* use sigstack for signals */
	int	u_sigslb;		/* lower bound of sigstack */
	stack_t	u_sigstack;		/* sp & on stack state variable */
#define	u_ssflags	u_sigstack.ss_flags
#define	u_sigsp		u_sigstack.ss_sp
#define	u_spsize	u_sigstack.ss_size
	ulong	u_suspmask;	/* real sig mask, temp. swapped in sigsuspend */
	int	u_restoremask;	/* boolean, does u_suspmask need replacing? */

	label_t	u_qsav;		/* label variable for quits and	*/
				/* interrupts			*/

	char	u_segflg;	/* IO flag: 0:user D; 1:system;	*/
				/*          2:user I		*/

	int	u_error;	/* return error code */

	ushort	u_uid;		/* effective user id */
	ushort	u_gid;		/* effective group id */
	ushort	u_ruid;		/* real user id */
	ushort	u_rgid;		/* real group id */
	gid_t	u_groups[NGROUPS_UMAX];   /* mult. grps, _NOGROUP terminates */

	struct proc *u_procp;	/* pointer to proc structure */

	int	*u_ap;		/* pointer to arglist */

	union {			/* syscall return values */
		struct	{
			int	r_val1;
			int	r_val2;
		}r_reg;
		off_t	r_off;
		time_t	r_time;
	} u_r;

	caddr_t	u_base;		/* base address for IO */
	unsigned u_count;	/* bytes remaining for IO */
	off_t	u_offset;	/* offset in file for IO */
	unsigned u_pbsize;	/* Bytes in block for IO */
	unsigned u_pboff;	/* offset in block for IO */
	short	u_fmode;	/* file mode for IO */
	dev_t	u_pbdev;	/* real device for IO */
	daddr_t	u_rablock;	/* read ahead block address */
	long	u_bsize;	/* block size of device */
	short	u_errcnt;	/* syscall error count */

	struct inode *u_cdir;	/* current directory */

	struct inode *u_rdir;	/* root directory */
	caddr_t	u_dirp;		/* pathname pointer */
	struct inode *u_pdir;	/* inode of parent directory of dirp */
	char	*u_tcptr;	/* pointer to tail component of last nami */
	off_t	u_scanpos;	/* offset into directory last scanned */
	ino_t	u_scaninum;	/* inum and dev of directory last scanned */
	dev_t	u_scandev;
	char	*u_comp;	/* pointer to current component */
	char	*u_nextcp;	/* pointer to beginning of next */

	int	u_arg[6];	/* arguments to current system call */

	unsigned u_orgtsize;	/* original text size (bytes) */
	unsigned u_orgdsize;	/* original data size (bytes) */
	unsigned u_orgbsize;	/* original bss size (bytes) */

	int	*u_ar0;		/* address of users saved R0 */

	struct	prof *u_profp;	/* pointer to list of prof structures */
	short	u_profn;	/* number of prof structures in list */

	long   u_execsz;

	char	u_comm[PSCOMSIZ];
	char	u_psargs[PSARGSZ];	/* arguments from exec system call */

	char	*u_tracepc;	/* Return PC if tracing enabled */
	int	u_sysabort;	/* Debugging: if set, abort syscall */
	int	u_systrap;	/* Are any syscall mask bits set? */ 
	sysset_t u_entrymask;	/* syscall stop-on-entry mask */
	sysset_t u_exitmask;	/* syscall stop-on-exit mask */
	fltset_t u_fltmask;	/* fault tracing mask for /proc */
	unsigned u_dbgflags;	/* addl. flags for low-freq /proc events */
	unsigned u_cursig;	/* current signal (used in psig and /debug) */
	struct dbinode *u_dbip; /* inode of most recent /proc open */

	time_t	u_start;
	time_t	u_ticks;
	unsigned long	u_mem;
	unsigned long	u_iosw;
	unsigned long	u_ioch;
	char	u_acflag;
	char	u_dorlimit;		/* Give Berkeley SIGXFSZ, when set */

	short	u_cmask;		/* mask for file creation */
	daddr_t	u_limit;		/* maximum write address */
	short	u_lock;			/* process/text locking flags */

	struct	itimerval u_timer[3];	/* interval timers */

	int	u_caddrflt;		/* index into table of functions to */
					/* handle user space external memory */
					/* faults encountered in kernel */
	int	u_flt_cause;		/* cause of external memory fault */
	int	u_flt_badvaddr;		/* vaddr of external memory fault */
	unsigned int	u_tlbcnt;	/* count of random TLB dropins	*/
					/* of second-level misses	*/
	struct file	**u_ofile;	/* Start of array of pointers	*/
					/* to file table entries for	*/
					/* open files.			*/
	char	*u_pofile;		/* Ptr to open file flag array.	 */
	struct	pathname *u_pathname;	/* pathname buffer for namei */

	caddr_t	u_eaaddr;		/* emulate area address */
	short	u_unblkonexecpid;	/* pid to unblock on exec */
	short	u_exitsig;		/* signal to send on exit of shgrp */
	int	u_code;			/* trap error code */
	struct monitor *u_monitor;	/* pointer to current monitor */
	union {
		pde_t	u_pde_ubptbl[NWIREDENTRIES-TLBWIREDBASE];
					/* page table entries for the	*/
					/* page tables, et al,	*/
					/* to avoid double misses */
		tlbpde_t u_tlbpde_ubptbl[NWIREDENTRIES-TLBWIREDBASE];
	} u_pde;
	caddr_t u_tlbhi_tbl[NWIREDENTRIES-TLBWIREDBASE];
					/* virtual addresses to go with */
					/* above page table		*/
	unsigned char	u_nexttlb;	/* index for next available	*/
					/* entry in above tables	*/
	ulong	u_tfaults;		/* # of 2nd level tfaults */
	ulong	u_vfaults;		/* # of vfaults */
	ulong	u_ufaults;		/* # of utlb misses */
	unsigned int	u_triggersave;	/* state of 3-way trigger hardware */
	struct rlimit u_rlimit[RLIM_NLIMITS];	/* 4.3BSD resource controls */
	seg_t	u_seglist[MAXPSEGS];	/* segment table */
	caddr_t	*u_segpage;		/* full segment table */
	/*
	 * Misc defines so that they now access fields in the rusage structs.
	 * Also declerations for PIOCUSAGE (/proc).
	 * Note that u_utime, u_stime, u_cutime and u_cstime reflect time in
	 * clock ticks (HZ), and not seconds.
	 */
#define u_utime		u_ru.ru_utime.tv_sec
#define u_stime		u_ru.ru_stime.tv_sec
#define u_ior		u_ru.ru_inblock
#define	u_iow		u_ru.ru_oublock
#define u_cutime	u_cru.ru_utime.tv_sec
#define u_cstime	u_cru.ru_stime.tv_sec
	struct rusage	u_ru;		/* stats for this proc */
	struct rusage	u_cru;		/* sum of stats for reaped children */
	u_long u_gbread;		/* # of of gigabytes of input */
	u_long u_bread;			/* # of bytes of input */
	u_long u_gbwrit;		/* # of of gigabytes of output */
	u_long u_bwrit;			/* # of bytes of output */
	u_long u_sysc;			/* # of syscalls */
	u_long u_syscr;			/* # of read system calls */
	u_long u_syscw;			/* # of write system calls */
	u_long u_syscps;		/* # of poll/select system calls */
	u_long u_sysci;			/* # of ioctl system calls */
	u_long u_graphfifo;		/* # of graphics pipeline stalls */
	u_long u_syscall;		/* current system call number */
        dev_t  u_ttyd;         		/* NOTE:used by SGI_RDUBLK only */
 
} user_t;


#ifdef _KERNEL
/*
* This "declaration" tells front end it can use r0 relative addressing.
*/
#define	u	(*(struct user *)UADDR)

/*
* Since u is strange, use up for debugging purposes.
*/
struct user *up;
#endif

#define u_exuid u_exfile.ux_uid
#define u_exgid u_exfile.ux_gid
#define u_exmode u_exfile.ux_mode

#define	u_rval1	u_r.r_reg.r_val1
#define	u_rval2	u_r.r_reg.r_val2
#define	u_roff	u_r.r_off
#define	u_rtime	u_r.r_time

#if R4000
#define	u_ubptbl	u_pde.u_tlbpde_ubptbl
#endif
#if R3000
#define	u_ubptbl	u_pde.u_pde_ubptbl
#endif

/* ioflag values: Read/Write, User/Kernel, Ins/Data */

#define	U_WUD	0
#define	U_RUD	1
#define	U_WKD	2
#define	U_RKD	3
#define	U_WUI	4
#define	U_RUI	5

#define	EXCLOSE	01

#ifdef MIPSEL
#define OBJMAGIC 0x0162
#endif
#ifdef MIPSEB
#define OBJMAGIC 0x0160
#endif

#endif /* __SYS_USER_H__ */
